@@ -25,18 +25,16 @@ import random
 import threading
 import platform
 import click
+import importlib
 
 
-try:
-    import view
-except:
-    from stuoe import view
+import view
 
 
 # Global Var
 verify_registered_email = list()
 ALLOWED_EXTENSIONS = {'txt', 'pdf', 'png', 'jpg', 'jpeg', 'gif'}
-Release = 'v0.1.3.6 Release'
+Release = 'v0.1.3.9 Release'
 
 
 # Get Configs File
@@ -65,10 +63,7 @@ manager = Manager(app)
 db = SQLAlchemy(app)
 mail = flask_mail.Mail(app)
 migrate = Migrate(app, db)
-manager.add_command('db', MigrateCommand) 
-
-
-
+manager.add_command('db', MigrateCommand)
 
 
 star_for = db.Table('star_for',
@@ -104,6 +99,21 @@ class User(db.Model):
     def __repr__(self):
         return {'id': self.id, 'email': self.email, 'user_des': self.user_des}
 
+    def theUserTalkNumber(self):
+        return len(Post.query.filter_by(pusher=self.id).all())
+
+    def theUserReplyNumber(self):
+        return len(Reply.query.filter_by(pusher=self.id).all())
+
+    def getStarNumber(self):
+        num = 0
+        for i in Post.query.filter_by(pusher=self.id).all():
+            num = num + len(i.star_user_list)
+        return num
+
+    def getActivity(self):
+        return Reply.query.filter_by(pusher=self.id).all()[:20] + Post.query.filter_by(pusher=self.id).all()[:20]
+
 
 class Group(db.Model):
     # Waiting....
@@ -133,27 +143,48 @@ class Post(db.Model):
     pushingtime = db.Column(db.Integer)
     tags = db.Column(db.Integer, db.ForeignKey('Tags.id'))
     lock = db.Column(db.Boolean, default='False')
+    look = db.Column(db.Integer, default='0')
     top = db.Column(db.Boolean, default='False')
     reply = db.relationship("Reply", backref="Post")
     star_user_list = db.relationship(
         'User', secondary=star_for, backref=db.backref('Post'))
+
     def getReplyNumber(self):
         return len(Reply.query.filter_by(father=self.id).all())
 
     def state(self):
-        reply = Reply.query.filter_by(father=self.id).first()
-        if not reply == None:
-            return User.query.filter_by(id=reply.pusher).first().nickname + ' 回复于 ' + Viewrender.getTimer(timetime=reply.pushingtime)
+        try:
+            reply = list(
+                reversed(Reply.query.filter_by(father=self.id).all()))[-1]
+        except BaseException:
+            reply = None
+        if reply is not None:
+            return User.query.filter_by(id=reply.pusher).first(
+            ).nickname + ' 回复于 ' + Viewrender.getTimer(timetime=reply.pushingtime)
         else:
-            return User.query.filter_by(id=self.pusher).first().nickname + ' 发布于 ' + Viewrender.getTimer(timetime=self.pushingtime)
+            return User.query.filter_by(id=self.pusher).first(
+            ).nickname + ' 发布于 ' + Viewrender.getTimer(timetime=self.pushingtime)
 
     def read(self):
-        self.look = look + 1
+        if self.look == None:
+            self.look = 0
+        self.look = int(self.look) + 1
         db.session.flush()
         db.session.commit()
         return self.look
 
-
+    def getParticipant(self):
+        ParticpantIdList = list()
+        Particpanter = list()
+        Particpanter.append(db_getuserByid(self.pusher))
+        ParticpantIdList.append(self.pusher)
+        for i in Reply.query.filter_by(father=self.id).all():
+            if i.pusher in ParticpantIdList:
+                pass
+            else:
+                Particpanter.append(db_getuserByid(i.pusher))
+                ParticpantIdList.append(i.pusher)
+        return Particpanter
 
 
 class Messages(db.Model):
@@ -203,7 +234,7 @@ if Group.query.filter_by(Group_name='管理员').first() is None:
         Highest_authority_group=True)
     db.session.add(AdminGourp)
     db.session.commit()
-if Tags.query.filter_by().first() == None:
+if Tags.query.filter_by().first() is None:
     if Tags.query.filter_by(name="新鲜事").first() is None:
         goodnewsTags = Tags(name='新鲜事', lock=False, icon='message')
         db.session.add(goodnewsTags)
@@ -327,6 +358,201 @@ def getPost_list(tags='', num=30):
     return returnPost_list
 
 
+# API Compment
+
+
+class Forum():
+    # init
+
+    def __init__(self):
+        self.app = app
+        self.db = db
+        self.serverconf = serverconf
+        self.view = Viewrender
+        self.get_session = get_session
+        self.some_sidebar = list()
+        self.loginuser_menu = list()
+        self.post_menu = list()
+        self.databaseTable = {
+            "User": User,
+            "Post": Post,
+            "Tags": Tags,
+            "star_for": star_for,
+            "File": File,
+            "Group": Group,
+            "Messages": Messages,
+            "Reply": Reply
+        }
+
+    # serverconf
+
+    # 得到配置文件的某个值,得到空字符串则返回整个配置文件
+
+    def serverconf_get(self, key):
+        if key == '':
+            return self.sevrerconf
+        else:
+            try:
+                return self.serverconf[key]
+            except BaseException:
+                return None
+
+    # 得到配置文件的某个值，不存在则创建为value，得到空字符串则返回整个配置文件
+    def serverconf_get_key_or_create(self, key, value):
+        if key == '':
+            return self.serverconf
+        else:
+            try:
+                return self.serverconf[key]
+            except BaseException:
+                self.serverconf[key] = value
+                self.amazing_fetch_to_serverconf()
+                return self.serverconf[key]
+
+    # 修改/创建配置文件的某个值
+    def serverconf_chang(self, key, value):
+        try:
+            self.serverconf[key] = value
+            self.amazing_fetch_to_serverconf()
+            return True
+        except BaseException:
+            return False
+
+    # database
+
+    # 得到整个db对象
+    def database_get_db(self):
+        return self.db
+
+    # 用修改的db替换掉db
+    def database_replace_database(self, db):
+        self.db = db
+
+    # 得到数据库中的所有表的immutabledict格式组
+
+    def database_get_all_table_immutabledict(self):
+        self.db.reflect(app=app)
+        return self.db.metadata.tables
+
+    # 得到数据库的表，不存在则为None
+    def database_get_table(self, tablename):
+        try:
+            return self.databaseTable[tablename]
+        except BaseException:
+            return None
+
+    # 添加模型到session
+    def database_add_to_session(self, models):
+        self.db.session.add(models)
+
+    # 添加模型到session并及时提交
+    def database_add_to_session_and_commit(self, models):
+        self.db.session.add(models)
+        self.db.session.flush()
+        self.db.session.commit()
+
+    # 删除模型
+    def database_delete_to_session(self, models):
+        self.db.session.delete(models)
+
+    # 直接提交
+
+    def database_commit(self):
+        self.db.session.flush()
+        self.db.session.commit()
+
+    # App and Route
+
+    # 得到整个app对象
+
+    def app_get_app(self):
+        return self.app
+
+    # 用修改的app替换掉app
+    def app_replace_app(self, app):
+        self.app = app
+
+    # view
+
+    # 发送一个标准的界面
+    def view_templates(self, auth, userObj, body, title):
+        return self.view.getTemplates(auth=auth, userObj=userObj, title=title, body=body)
+
+    # 检查用户
+    def view_check_user(self, type):
+        return self.get_session(type)
+
+    # 增加首页侧边栏(默认登入状态下显示)
+    def view_sidebar_add(self, name, url, icon):
+        self.some_sidebar.append({
+            "name": name,
+            "icon": icon,
+            "url": url
+        })
+    
+    # 增加登入用户右上角个人菜单选项，从“资料”和已添加控件后开始增加
+    def view_loginuser_menu_add_option(self,name,url,icon):
+        self.loginuser_menu.append({
+            "type":"options",
+            "name":name,
+            "icon":icon,
+            "url":url
+        })
+    
+    # 增加登入用户右上角个人菜单分割线，从“资料”和已添加控件后开始增加
+    def view_loginuser_menu_add_line(self):
+        self.post_menu.append({
+            "type":"line"
+        })
+    
+    # 增加登入用户查看帖子右下角菜单选项，从“设为星标”和已添加控件后开始增加
+    def view_post_menu_add_option(self,name,url,icon):
+        self.post_menu.append({
+            "type":"options",
+            "name":name,
+            "icon":icon,
+            "url":url
+        })
+    
+    # 增加登入用户查看帖子右下角菜单分割线，从“设为星标”和已添加控件后开始增加
+    def view_post_menu_add_line(self):
+        self.post_menu.append({
+            "type":"line"
+        })
+
+    # Amazing Fetch
+
+    def amazing_fetch(self):
+        global app
+        global serverconf
+        global db
+        app = self.app
+        serverconf = self.serverconf
+        db = self.db
+        Viewrender.forum = self
+
+    def amazing_fetch_to_serverconf(self):
+        open('server.conf', 'w+', encoding="utf-8").write(str(self.serverconf))
+        self.serverconf = dict(eval(open('server.conf', 'rb').read()))
+        self.view.serverconf = self.serverconf
+
+
+extensionlist = []
+
+forum = Forum()
+
+# Import Extensions
+# 丢手绢！
+for i in os.listdir("extension"):
+    print("=============================")
+    print("导入模块: " + i)
+    print("=============================\n")
+    theExtensions = importlib.import_module("extension." + i + ".main")
+    extensionlist.append(theExtensions.header)
+    forum = theExtensions.Main(forum).init(forum=forum)
+
+
+forum.amazing_fetch()
 
 
 # Install
@@ -388,18 +614,67 @@ def installing_step():
 def send_index():
 
     if get_session() == False:
-        return Viewrender.gethome(auth=False, tagslist=Tags.query.filter_by().all(), postlist=getPost_list(), get_avater=get_avater,title="首页")
+        return Viewrender.gethome(
+            auth=False,
+            tagslist=Tags.query.filter_by().all(),
+            postlist=getPost_list(),
+            get_avater=get_avater,
+            title="首页",
+            options='index')
     else:
         try:
             serverconf['open_email']
-        except:
+        except BaseException:
             serverconf['open_email'] = False
             open('server.conf', 'w+', encoding='utf-8').write(str(serverconf))
         if not serverconf['open_email']:
             get_session('obj').verify_email = True
             db.session.flush()
             db.session.commit()
-        return Viewrender.gethome(auth=True, userObj=get_session('obj'), tagslist=Tags.query.filter_by().all(), postlist=getPost_list(), get_avater=get_avater,title="首页")
+        return Viewrender.gethome(
+            auth=True,
+            userObj=get_session('obj'),
+            tagslist=Tags.query.filter_by().all(),
+            postlist=getPost_list(),
+            get_avater=get_avater,
+            title="首页",
+            options='index')
+
+
+@app.route('/my')
+def my_post():
+    user = get_session('obj')
+    if not user:
+        return abort(403)
+    return Viewrender.gethome(
+        auth=True,
+        userObj=get_session('obj'),
+        tagslist=Tags.query.filter_by().all(),
+        postlist=Post.query.filter_by(
+            pusher=user.id).all(),
+        get_avater=get_avater,
+        title="首页",
+        options='my')
+
+
+@app.route('/notifications')
+def user_notifications():
+    user = get_session('obj')
+    if not user:
+        return abort(403)
+    theUserAllMessages = Messages.query.filter_by(Postman=user.id).all()
+    renderNoticPages = jinja2.Template(
+        open(
+            'storage/templates/box.html',
+            'r',
+            encoding="utf-8").read()).render(
+        msglist=theUserAllMessages,
+        tagslist=Tags.query.filter_by().all())
+    return Viewrender.getTemplates(
+        auth=True,
+        userObj=user,
+        title='通知',
+        body=renderNoticPages)
 
 
 @app.route('/logout')
@@ -417,10 +692,14 @@ def user_space(uid):
     )[:3] + Post.query.filter_by(pusher=user.id).all()[:3]
     lookuser = get_session('obj')
     if not lookuser:
-        return Viewrender.getUserSpace(auth=False, lastedPost=lastedPost, lookuserObj=user)
+        return Viewrender.getUserSpace(
+            auth=False, lastedPost=lastedPost, lookuserObj=user)
     else:
         return Viewrender.getUserSpace(
-            auth=True, lookuserObj=user, userObj=lookuser, lastedPost=lastedPost)
+            auth=True,
+            lookuserObj=user,
+            userObj=lookuser,
+            lastedPost=lastedPost)
 
 
 @app.route('/p/<pid>')
@@ -470,13 +749,24 @@ def post_stared(pid):
 
 @app.route('/t/<tid>')
 def get_tags(tid):
-    if Tags.query.filter_by(id=tid).first() == None:
+    if Tags.query.filter_by(id=tid).first() is None:
         return abort(404)
     tagsname = Tags.query.filter_by(id=tid).first().name
     if get_session() == False:
-        return Viewrender.gethome(auth=False, tagslist=Tags.query.filter_by().all(), postlist=getPost_list(tags=tid), get_avater=get_avater,title=tagsname)
+        return Viewrender.gethome(
+            auth=False,
+            tagslist=Tags.query.filter_by().all(),
+            postlist=getPost_list(tagsname, 40),
+            get_avater=get_avater,
+            title=tagsname)
     else:
-        return Viewrender.gethome(auth=True, userObj=get_session('obj'), tagslist=Tags.query.filter_by().all(), postlist=getPost_list(tags=tid), get_avater=get_avater,title=tagsname)
+        return Viewrender.gethome(
+            auth=True,
+            userObj=get_session('obj'),
+            tagslist=Tags.query.filter_by().all(),
+            postlist=getPost_list(tagsname, 40),
+            get_avater=get_avater,
+            title=tagsname)
 
 
 @app.route('/relation')
@@ -490,9 +780,16 @@ def show_relation():
             if i.pusher == nowUserId:
                 relationPost.append(i)
             else:
-                for i in Reply.query.filter_by(father=i.id,pusher=nowUserId).all():
+                for i in Reply.query.filter_by(
+                        father=i.id, pusher=nowUserId).all():
                     relationPost.append(i)
-        return Viewrender.gethome(auth=True, userObj=get_session('obj'), tagslist=Tags.query.filter_by().all(), postlist=relationPost, get_avater=get_avater,title="与我有关")
+        return Viewrender.gethome(
+            auth=True,
+            userObj=get_session('obj'),
+            tagslist=Tags.query.filter_by().all(),
+            postlist=relationPost,
+            get_avater=get_avater,
+            title="与我有关")
 
 
 @app.route('/write')
@@ -574,7 +871,8 @@ def user_changsettings_checkemail():
                 return Viewrender.getMSG("验证码不正确", auth=True, userObj=user)
     return Viewrender.getMSG('该用户并未发起更改邮件事务,找不到对象', auth=True, userObj=user)
 
-@app.route('/settings/password',methods=["POST"])
+
+@app.route('/settings/password', methods=["POST"])
 def user_changsettings_password():
     user = get_session('obj')
     if not user:
@@ -585,21 +883,17 @@ def user_changsettings_password():
     if request.form['newpassword'] == '':
         return Viewrender.getMSG("密码不能为空")
     if request.form['newpassword'] != request.form['againpassword']:
-        return Viewrender.getMSG("新的密码并不一致",auth=True,userObj=user)
+        return Viewrender.getMSG("新的密码并不一致", auth=True, userObj=user)
     passhash = hashlib.sha256(
         request.form['oldpassword'].encode('utf-8')).hexdigest()
     if not user.passhash == passhash:
-        return Viewrender.getMSG("旧密码不正确",auth=True,userObj=user)
+        return Viewrender.getMSG("旧密码不正确", auth=True, userObj=user)
     passhash = hashlib.sha256(
         request.form['newpassword'].encode('utf-8')).hexdigest()
     user.passhash = passhash
     db.session.flush()
     db.session.commit()
-    return Viewrender.getMSG("设置成功",auth=True,userObj=user)
-
-
-
-
+    return Viewrender.getMSG("设置成功", auth=True, userObj=user)
 
 
 @app.route('/settings/avater', methods=['POST'])
@@ -609,7 +903,8 @@ def uploader_avater():
         return abort(403)
     avater = request.files['avater']
     basepath = os.path.dirname(__file__)
-    upload_path = os.getcwd() + '/CacheFile/' + str(random.randint(100000,9999999)) + "__file__" + secure_filename(avater.filename)
+    upload_path = os.getcwd() + '/CacheFile/' + str(random.randint(100000, 9999999)
+                                                    ) + "__file__" + secure_filename(avater.filename)
     avater.save(upload_path)
     avaterData = open(upload_path, 'rb').read()
     avaterFilename = os.path.basename(upload_path)
@@ -641,7 +936,7 @@ def adminSettings(pages):
         return abort(403)
     if not user.user_group == '管理员':
         return abort(403)
-    if not pages in ['preview', 'profile', 'tags', 'style', 'extension']:
+    if pages not in ['preview', 'profile', 'tags', 'style', 'extension']:
         return abort(404)
     if pages == 'preview':
         replyList = list()
@@ -649,33 +944,73 @@ def adminSettings(pages):
             replyList.append(replyObj(user=db_getuserByid(i.pusher), reply=i))
         adminlist = open('storage/templates/admin/list.html',
                          'r', encoding="utf-8").read()
-        body = jinja2.Template(open('storage/templates/admin/preview.html', 'r', encoding="utf-8").read()).render(adminList=adminlist, pv=platform.python_version(),
-                                                                                                                  fv=__version__, sv=Release, postlist=list(reversed(Post.query.filter_by().all()[:7])), get_avater=get_avater, replyList=replyList)
-        return Viewrender.getTemplates(title='管理界面', auth=True, base2=True, body=body, userObj=user)
+        body = jinja2.Template(
+            open(
+                'storage/templates/admin/preview.html',
+                'r',
+                encoding="utf-8").read()).render(
+            adminList=adminlist,
+            pv=platform.python_version(),
+            fv=__version__,
+            sv=Release,
+            postlist=list(
+                reversed(
+                    Post.query.filter_by().all()[
+                        :7])),
+            get_avater=get_avater,
+            replyList=replyList)
+        return Viewrender.getTemplates(
+            title='管理界面', auth=True, base2=True, body=body, userObj=user)
     if pages == 'profile':
         adminlist = open('storage/templates/admin/list.html',
                          'r', encoding="utf-8").read()
-        body = jinja2.Template(open('storage/templates/admin/profile.html', 'r',
-                                    encoding="utf-8").read()).render(adminList=adminlist, serverconf=serverconf)
-        return Viewrender.getTemplates(title='管理界面', auth=True, base2=True, body=body, userObj=user)
+        body = jinja2.Template(
+            open(
+                'storage/templates/admin/profile.html',
+                'r',
+                encoding="utf-8").read()).render(
+            adminList=adminlist,
+            serverconf=serverconf)
+        return Viewrender.getTemplates(
+            title='管理界面', auth=True, base2=True, body=body, userObj=user)
     if pages == 'style':
         adminlist = open('storage/templates/admin/list.html',
                          'r', encoding="utf-8").read()
-        body = jinja2.Template(open('storage/templates/admin/style.html', 'r',
-                                    encoding="utf-8").read()).render(adminList=adminlist, serverconf=serverconf)
-        return Viewrender.getTemplates(title='管理界面', auth=True, base2=True, body=body, userObj=user)
+        body = jinja2.Template(
+            open(
+                'storage/templates/admin/style.html',
+                'r',
+                encoding="utf-8").read()).render(
+            adminList=adminlist,
+            serverconf=serverconf)
+        return Viewrender.getTemplates(
+            title='管理界面', auth=True, base2=True, body=body, userObj=user)
     if pages == 'tags':
         adminlist = open('storage/templates/admin/list.html',
                          'r', encoding="utf-8").read()
-        body = jinja2.Template(open('storage/templates/admin/tags.html', 'r',
-                                    encoding="utf-8").read()).render(adminList=adminlist, serverconf=serverconf, tagslist=Tags.query.filter_by().all())
-        return Viewrender.getTemplates(title='管理界面', auth=True, base2=True, body=body, userObj=user)
+        body = jinja2.Template(
+            open(
+                'storage/templates/admin/tags.html',
+                'r',
+                encoding="utf-8").read()).render(
+            adminList=adminlist,
+            serverconf=serverconf,
+            tagslist=Tags.query.filter_by().all())
+        return Viewrender.getTemplates(
+            title='管理界面', auth=True, base2=True, body=body, userObj=user)
     if pages == 'extension':
         adminlist = open('storage/templates/admin/list.html',
                          'r', encoding="utf-8").read()
-        body = jinja2.Template(open('storage/templates/admin/ext.html', 'r',
-                                    encoding="utf-8").read()).render(adminList=adminlist, serverconf=serverconf, tagslist=Tags.query.filter_by().all())
-        return Viewrender.getTemplates(title='管理界面', auth=True, base2=True, body=body, userObj=user)
+        body = jinja2.Template(
+            open(
+                'storage/templates/admin/ext.html',
+                'r',
+                encoding="utf-8").read()).render(
+            adminList=adminlist,
+            serverconf=serverconf,
+            extensionlist=extensionlist)
+        return Viewrender.getTemplates(
+            title='管理界面', auth=True, base2=True, body=body, userObj=user)
 
 
 @app.route('/adminwait/profile', methods=['POST'])
@@ -729,10 +1064,13 @@ def adminWait_tid(tid):
     tagsObj = Tags.query.filter_by(id=tid).first()
     if not tagsObj:
         if tid == 'new':
-            if not Tags.query.filter_by(name=request.form['tagsname']).first() == None:
+            if not Tags.query.filter_by(
+                    name=request.form['tagsname']).first() is None:
                 return Viewrender.getMSG(msg='名称重复', auth=True, userObj=user)
             newTags = Tags(
-                name=request.form['tagsname'], icon=request.form['tagsicon'], lock=False)
+                name=request.form['tagsname'],
+                icon=request.form['tagsicon'],
+                lock=False)
             db.session.add(newTags)
             db.session.flush()
             db.session.commit()
@@ -746,24 +1084,49 @@ def adminWait_tid(tid):
     return redirect('/admin/tags')
 
 
+# 操作区
+
+
+# 删除帖子
 @app.route('/rmpost/<pid>')
 def rmpost(pid):
     user = get_session('obj')
+    post = Post.query.filter_by(id=pid).first()
+    if post is None:
+        return Viewrender.getMSG(auth=True, userObj=user, msg='帖子不存在')
+    if user is None:
+        return abort(403)
+    if (user.id == int(post.pusher)) or (user.user_group == '管理员'):
+        for i in Reply.query.filter_by(father=pid).all():
+            db.session.delete(i)
+            db.session.flush()
+            db.session.commit()
+        db.session.delete(post)
+        db.session.flush()
+        db.session.commit()
+        return redirect('/')
+    return abort(403)
+
+
+# 删除回复
+@app.route('/rmreply/<rid>')
+def rmreply(rid):
+    user = get_session('obj')
+    reply = Reply.query.filter_by(id=rid).first()
+    red = reply.father
     if not user:
         return abort(403)
-    if not user.user_group == '管理员':
+    if reply is None:
+        return Viewrender.getMSG(auth=True, userObj=user, msg='回复不存在')
+    if not ((user.user_group == '管理员') or (user.id == int(reply.pusher))):
         return abort(403)
-    post = Post.query.filter_by(id=pid).first()
-    if post == None:
-        return Viewrender.getMSG(auth=True, userObj=user, msg='帖子不存在')
-    for i in Reply.query.filter_by(father=pid).all():
-        db.session.delete(i)
-    db.session.delete(post)
+    db.session.delete(reply)
     db.session.flush()
     db.session.commit()
-    return redirect('/')
+    return redirect('/p/' + str(red))
 
 
+# 锁定帖子
 @app.route('/lock/<pid>')
 def lock(pid):
     user = get_session('obj')
@@ -772,13 +1135,15 @@ def lock(pid):
     if not user.user_group == '管理员':
         return abort(403)
     post = Post.query.filter_by(id=pid).first()
-    if post == None:
+    if post is None:
         return Viewrender.getMSG(auth=True, userObj=user, msg='帖子不存在')
     post.lock = True
     db.session.flush()
     db.session.commit()
     return redirect('/p/' + str(pid))
 
+# 解除锁定帖子
+
 
 @app.route('/unlock/<pid>')
 def unlock(pid):
@@ -788,13 +1153,15 @@ def unlock(pid):
     if not user.user_group == '管理员':
         return abort(403)
     post = Post.query.filter_by(id=pid).first()
-    if post == None:
+    if post is None:
         return Viewrender.getMSG(auth=True, userObj=user, msg='帖子不存在')
     post.lock = False
     db.session.flush()
     db.session.commit()
     return redirect('/p/' + str(pid))
 
+# 顶置帖子
+
 
 @app.route('/top/<pid>')
 def top(pid):
@@ -804,13 +1171,15 @@ def top(pid):
     if not user.user_group == '管理员':
         return abort(403)
     post = Post.query.filter_by(id=pid).first()
-    if post == None:
+    if post is None:
         return Viewrender.getMSG(auth=True, userObj=user, msg='帖子不存在')
     post.top = True
     db.session.flush()
     db.session.commit()
     return redirect('/p/' + str(pid))
 
+# 取消顶置帖子
+
 
 @app.route('/untop/<pid>')
 def untop(pid):
@@ -820,47 +1189,99 @@ def untop(pid):
     if not user.user_group == '管理员':
         return abort(403)
     post = Post.query.filter_by(id=pid).first()
-    if post == None:
+    if post is None:
         return Viewrender.getMSG(auth=True, userObj=user, msg='帖子不存在')
     post.top = False
     db.session.flush()
     db.session.commit()
     return redirect('/p/' + str(pid))
 
+# 删除消息
 
+
+@app.route('/rmmsg/<mid>')
+def rmmsg(mid):
+    user = get_session('obj')
+    msg = Messages.query.filter_by(id=mid).first()
+    if not user:
+        return abort(403)
+    if msg is None:
+        return Viewrender.getMSG(auth=True, userObj=user, msg='消息不存在')
+    if not ((user.user_group == '管理员') or (user.id == int(msg.Postman))):
+        return abort(403)
+    db.session.delete(msg)
+    db.session.flush()
+    db.session.commit()
+    return redirect('/notifications')
+
+# 取消帖子星标
+
+
+@app.route("/unstar/<pid>")
+def unstar(pid):
+    obj = db_getpostByid(pid)
+    if obj is None:
+        return abort(404)
+    user = get_session('obj')
+    if not user:
+        return abort(403)
+    star_for_table = star_for.query.filter_by(
+        star_user=user.id, stared_post=obj.id).first()
+    if star_for_table == None:
+        return abort(400)
+    else:
+        db.session.delete(star_for_table)
+        db.session.flush()
+        db.session.commit()
+        return redirect("/p/" + str(obj.id))
+
+
+# 搜索列
 class SearchObj():
     def __init__(self, avater, url, name):
         self.avater = avater
         self.url = url
         self.name = name
 
+# 搜索
+
 
 @app.route('/search', methods=['GET', 'POST'])
 def getSearch():
     user = get_session('obj')
-    auth = not (user == None)
+    auth = not (user is None)
     if request.method == 'POST':
         SearchList = []
         SearchText = request.form['SearchText']
         for i in User.query.filter_by().all():
-            if (SearchText in i.nickname) or (SearchText == i.id) or (SearchText in i.user_des):
-                SearchList.append(
-                    SearchObj(avater=i.avater, name="用户: " + str(i.nickname), url="/u/" + str(i.id)))
+            if (SearchText in i.nickname) or (
+                    SearchText == i.id) or (SearchText in i.user_des):
+                SearchList.append(SearchObj(
+                    avater=i.avater, name="用户: " + str(i.nickname), url="/u/" + str(i.id)))
         for i in Post.query.filter_by().all():
             if (SearchText in i.title) or (SearchText in i.body):
                 SearchList.append(SearchObj(avater=get_avater(
                     i.pusher), name="讨论: " + i.title, url="/p/" + str(i.id)))
         user = get_session('obj')
         body = jinja2.Template(
-            open('storage/templates/search.html', 'r', encoding="utf-8").read()).render(SearchList=SearchList)
+            open(
+                'storage/templates/search.html',
+                'r',
+                encoding="utf-8").read()).render(
+            SearchList=SearchList)
 
-        return Viewrender.getTemplates(auth=auth, userObj=user, title='搜索', body=body)
+        return Viewrender.getTemplates(
+            auth=auth, userObj=user, title='搜索', body=body)
     else:
 
         body = jinja2.Template(
-            open('storage/templates/search.html', 'r', encoding="utf-8").read()).render()
+            open(
+                'storage/templates/search.html',
+                'r',
+                encoding="utf-8").read()).render()
 
-        return Viewrender.getTemplates(auth=auth, userObj=user, title='搜索', body=body)
+        return Viewrender.getTemplates(
+            auth=auth, userObj=user, title='搜索', body=body)
 
 
 # Staticfile
@@ -869,7 +1290,7 @@ def getSearch():
 @app.route('/dynamic/<fid>/<obj>')
 def send_dynamice(fid, obj):
     fileObj = File.query.filter_by(id=fid).first()
-    if fileObj == None:
+    if fileObj is None:
         return abort(404)
     return fileObj.fileData
 
@@ -962,6 +1383,7 @@ def pushing_post():
         pushingtime=time.time(),
         tags=request.form['tags'],
         lock=False,
+        look=0,
         top=False)
     db.session.add(newPost)
     db.session.flush()
@@ -992,9 +1414,105 @@ def make_Reply(pid):
     db.session.add(newReply)
     db.session.flush()
     db.session.commit()
+    makeNotice(
+        type='newReply',
+        info={
+            'postId': pid,
+            'ReplyUserID': user.id,
+            'newReplyId': newReply.id})
     return redirect('/p/' + str(post.id))
 
 
+# 400,403,404,405,500 Pages For Error
+
+def get_user_something():
+    if get_session('obj') == False:
+        return False
+    return True
+
+# FlaskError默认返回响应头是200，这里需要调整
+
+
+def make_response_for_error(html, e):
+    response = make_response(html)
+    response.status = e
+    return response
+
+
+@app.errorhandler(400)
+def error_400(e):
+    auth = get_user_something()
+    return make_response_for_error(
+        Viewrender.getTemplates(
+            body=open(
+                "storage/templates/error/400.html",
+                'r',
+                encoding="utf-8").read(),
+            auth=auth,
+            userObj=get_session('obj'),
+            title="400 恶意请求"),
+        "400")
+
+
+@app.errorhandler(403)
+def error_403(e):
+    auth = get_user_something()
+    return make_response_for_error(
+        Viewrender.getTemplates(
+            body=open(
+                "storage/templates/error/403.html",
+                'r',
+                encoding="utf-8").read(),
+            auth=auth,
+            userObj=get_session('obj'),
+            title="403 无权限访问"),
+        "403")
+
+
+@app.errorhandler(404)
+def error_404(e):
+    auth = get_user_something()
+    return make_response_for_error(
+        Viewrender.getTemplates(
+            body=open(
+                "storage/templates/error/404.html",
+                'r',
+                encoding="utf-8").read(),
+            auth=auth,
+            userObj=get_session('obj'),
+            title="404 界面不存在"),
+        "404")
+
+
+@app.errorhandler(405)
+def error_405(e):
+    auth = get_user_something()
+    return make_response_for_error(
+        Viewrender.getTemplates(
+            body=open(
+                "storage/templates/error/405.html",
+                'r',
+                encoding="utf-8").read(),
+            auth=auth,
+            userObj=get_session('obj'),
+            title="405 请求错误"),
+        "405")
+
+
+@app.errorhandler(500)
+def error_500(e):
+    auth = get_user_something()
+    pages500 = jinja2.Template(open(
+        "storage/templates/error/500.html", 'r', encoding="utf-8").read()).render(error=e)
+    return make_response_for_error(
+        Viewrender.getTemplates(
+            body=pages500,
+            auth=auth,
+            userObj=get_session('obj'),
+            title="500 服务器错误"),
+        "500")
+
+
 def send_mail(msg):
     with app.app_context():
         mail.send(msg)
@@ -1015,24 +1533,76 @@ def get_fileUrl(fileObj, id):
 
 def get_avater(userId):
     obj = User.query.filter_by(id=userId).first()
-    if obj == None:
+    if obj is None:
         return 'None'
     return obj.avater
 
 
-def postNotice(userId, title, body, user):
-    newNotice = Messages(subject=title, body=body, PostTime=time.time(), Postman=userId,
-                         avater='http://identicon.relucks.org/' + str(random.randint(200, 999)) + '?size=120')
-    msg = flask_mail.Message(recipients=db_getuserByid(userId).email,
-                             html=body,
-                             subject='网站通知:' + title,)
-    threading._start_new_thread(send_mail, (msg,))
-    current_app.logger.info(
-        "向" + db_getuserByid(userId).email + "发送了一个通知 标题:" + title + " 内容:" + body)
-
+def postNotice(userId, title, body):
+    newNotice = Messages(
+        subject=title,
+        body=body,
+        PostTime=time.time(),
+        Postman=userId,
+        avater='http://identicon.relucks.org/' +
+        str(
+            random.randint(
+                200,
+                999)) +
+        '?size=120')
+    db.session.add(newNotice)
+    db.session.flush()
+    db.session.commit()
     return redirect('/settings/check')
 
 
+def makeNotice(type, info):
+    # 所有通知的人
+    NoticGroupId = list()
+    NoticGroup = list()
+    NoticGroupEmail = list()
+    if type == 'newReply':
+        thePost = Post.query.filter_by(id=info['postId']).first()
+        # 通知所有的参与者
+        for i in thePost.getParticipant():
+            NoticGroup.append(i)
+            NoticGroupId.append(i.id)
+            NoticGroupEmail.append(i.email)
+        # 通知所有的关注者（排除参与者）
+        for i in thePost.star_user_list:
+            if i.id in NoticGroupId:
+                pass
+            else:
+                NoticGroup.append(i)
+                NoticGroupId.append(i.id)
+                NoticGroupEmail.append(i.email)
+        # 生产消息
+        NoticReplyUser = info['ReplyUserID']
+        # 推送到所有参与者和关注者
+        for i in NoticGroupId:
+            postNotice(
+                userId=i,
+                title=thePost.title + '中的新回复',
+                body=Reply.query.filter_by(
+                    id=info['newReplyId']).first().body)
+        # 发生邮件
+        msg = flask_mail.Message(recipients=NoticGroupEmail, html=Viewrender.m2(Reply.query.filter_by(
+            id=info['newReplyId']).first().body,),body=Viewrender.m2(Reply.query.filter_by(
+            id=info['newReplyId']).first().body,) ,subject='参与或星标的讨论有新回复',)
+        threading._start_new_thread(send_mail, (msg,))
+
+
+def renderNoticXML(msglist):
+    xml = ''
+    for i in msglist:
+        xml = xml + jinja2.Template(open('storage/templates/xml/notic.html',
+                                         'r', encoding="utf-8").read()).render(title=i.subject, body=i.body)
+    return xml
+
+
 @ app.route('/robots.txt')
 def robotsTxt():
     return serverconf['robots.txt']
+
+
+app.run(debug=False, host="0.0.0.0", port=5000)
